# Copyright 2025 Scientific Knowledge Organization (SciKnowOrg) Research Group.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from pathlib import Path

# Load version from VERSION file
__version__ = (Path(__file__).parent / "VERSION").read_text().strip()

from .pipeline import OntoAlignerPipeline
from ontoaligner import ontology, base, encoder, aligner, utils, postprocess

__all__ = [
    "ontology",
    "base",
    "encoder",
    "aligner",
    "utils",
    "postprocess",
    "OntoAlignerPipeline"
]
